#!/usr/bin/env python
#Author: Mark Feineigle
#Create Date: 2018/05/03
#Last Modify: 2018/05/03
# deeper learning insights machine learning page 638

from time import time
import numpy as np
import matplotlib.pyplot as plt
from sklearn.datasets import load_digits
from sklearn.preprocessing import scale
from sklearn.cluster import KMeans
from sklearn import metrics
np.random.seed()

digits = load_digits()
data = scale(digits.data)
n_samples, n_features = data.shape
n_digits = len(np.unique(digits.target))
labels = digits.target
sample_size = 300

print("n_digits: %d, \t n_samples %d, \t n_features %d" % (n_digits, n_samples, n_features))
print(79 * '_')
print('% 9s' % 'init ARI AMI silhouette time inertia homo compl v-meas')

def bench_k_means(estimator, name, data):
  t0 = time()
  estimator.fit(data)
  print('% 9s %.2fs %i %.3f %.3f %.3f %.3f %.3f '
  % (name,
    (time() - t0),
    estimator.inertia_,
    metrics.homogeneity_score(labels, estimator.labels_),
    metrics.completeness_score(labels, estimator.labels_),
    metrics.v_measure_score(labels, estimator.labels_),
    metrics.adjusted_rand_score(labels, estimator.labels_),
    metrics.silhouette_score(data, estimator.labels_,
                             metric='euclidean',
                             sample_size=sample_size)))

bench_k_means(estimator=KMeans(init='k-means++',
                               n_clusters=n_digits,
                               n_init=10),
              name="k-means++",
              data=data)
print(79 * '_')

from sklearn.decomposition import PCA
pca = PCA(n_components=n_digits).fit(data)
bench_k_means(KMeans(init=pca.components_, n_clusters=10),
              name="PCA-based",
              data=data)


# Elbow method
import numpy as np
from sklearn.cluster import KMeans
from sklearn.datasets import load_digits
from scipy.spatial.distance import cdist
import matplotlib.pyplot as plt
from sklearn.decomposition import PCA
from sklearn.preprocessing import scale

digits = load_digits()
data = scale(digits.data)
n_samples, n_features = data.shape
n_digits = len(np.unique(digits.target))
labels = digits.target

K = range(1,20)
explainedvariance= []
for k in K:
  reduced_data = PCA(n_components=2).fit_transform(data)
  kmeans = KMeans(init = 'k-means++', n_clusters = k, n_init = k)
  kmeans.fit(reduced_data)
  explainedvariance.append(sum(np.min(cdist(reduced_data, 
                                      kmeans.cluster_centers_, 'euclidean'),
                                      axis=1))/data.shape[0])
plt.plot(K, explainedvariance, 'bx-')
plt.show()
 

# K-folds cross validation
import numpy as np
from sklearn import cross_validation
from sklearn.cluster import KMeans
from sklearn.datasets import load_digits
from sklearn.preprocessing import scale

digits = load_digits()
data = scale(digits.data)
n_samples, n_features = data.shape
n_digits = len(np.unique(digits.target))
labels = digits.target

kmeans = KMeans(init='k-means++', n_clusters=n_digits, n_init=n_digits)
cv = cross_validation.ShuffleSplit(n_samples, n_iter=10, test_size=0.4, random_state=0)
scores = cross_validation.cross_val_score(kmeans, data, labels, cv=cv, scoring='adjusted_rand_score')
print(scores)
print(sum(scores)/cv.n_iter)
 
